# frozen_string_literal: true

module Hermes
  # LessorObjectives model
  class LessorObjective
    include Hermes::Concerns::CommonFieldsConcern
    require Rails.root.join(
      'lib',
      'core_extentions',
      'parameters',
      'handle_date'
    ).to_s
    hermes_tools 'typology'
    
    FILTER_KEYS = [
=begin
      department,
      district,
      threshold_type,
      priority_public,
      already_inhabited,
      operation_name,
=end
      "agency_id"
    ]
    
    field :activation_date, type: Date
    
    field :department, type: String
    field :district, type: String
    field :threshold_type, type: String
    field :priority_public, type: Boolean
    field :new_offer, type: Boolean
    field :operation_name, type: String
    field :agency_id, type: BSON::ObjectId
    
    field :target_by_typologies, type: Hash, default: {}
    field :present_typologies, type: Array, default: []
    field :realized_by_typologies, type: Hash, default: {}
    field :objective_difference_by_typologies, type: Hash, default: {}
    field :test, type: String
    
    field :lessor_id, type: BSON::ObjectId
    belongs_to :lessor, class_name: 'Hermes::Lessor', foreign_key: :lessor_id, primary_key: :id, index: true
    has_many :lessor_objective_histories, class_name: 'Hermes::LessorObjectiveHistory'
    has_and_belongs_to_many :housing_offer, class_name: 'Hermes::HousingOffer'
    
    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id
    
    scope :actives, -> {
      first_of_january = DateTime.new(Date.current.year, 1, 1, 0, 0, 0).utc
      where(
        activation_date: {
          :$gte => first_of_january,
          :$lte => first_of_january + 1.year
        }
      )
    }
    
    common_indexes *%i[
      department
      district
      threshold_type
      priority_public
      new_offer
      operation_name
      agency_id
      lessor_id
      lessor_objective_history_ids
      activation_date
    ]
    scope :with_new, -> {where(new_offer: true)}
    
    before_create :before_create_actions
    before_update :before_actions
    before_save :before_save_actions
    after_save :after_save_actions
    
    
    def targeted_offers_count
      Hermes::HousingOffer.will_trigger_objective(id).count
    end
    
    def agency
      Hermes::Entity.find_by(id: agency_id)
    end
    
    def lessor_id=(new_lessor_id)
      now = DateTime.current
      previous_lessor = Hermes::Lessor.find_by(id: lessor_id)
      new_lessor = Hermes::Lessor.find_by(id: new_lessor_id)
      previous_lessor&.set(lessor_objective_ids: (previous_lessor&.lessor_objective_ids || []) - [id], updated_at: now)
      new_lessor&.set(lessor_objective_ids: (new_lessor&.lessor_objective_ids || []) + [id], updated_at: now)
      super(new_lessor_id)
    end
    
    def lessor_id
      lessor&.id
    end
    
    def sum_by_typologies_for(label)
      field_label = "#{label}_by_typologies"
      field_value = send(field_label)
      return 0 unless field_value.present?
      
      values = field_value.values.map(&:to_i).compact
      return 0 unless values.present?
      
      values.sum
    end
    
    def handle_activation_date
      self.activation_date = DateTime.current.utc unless activation_date
    end
    
    def correct_target_by_typologies
      return if target_by_typologies.present?
      
      self.target_by_typologies = target_by_typologies.each_with_object({}) do |(key, val), h|
        key_is_valid = (key == 'all' || key.is_typology?)
        val_is_number = %w[Integer Number].include?(val.class.name)
        h[key] = val.to_i if key_is_valid && val_is_number
      end
    end
    
    def update_present_typologies
      return false unless target_by_typologies.present?
      
      self.present_typologies = if target_by_typologies[:all] || target_by_typologies['all']
                                  ['all']
                                else
                                  target_by_typologies.keys - [:all, 'all']
                                end
    end
    
    def lessor_status_filter
      first_of_january = Date.parse("#{activation_date.year}-01-01")
      {
        lessor_status: {:$in => HousingOffer.rented_statuses},
        lessor_status_updated_at: {:$gte => first_of_january}
      }
    end
    
    def typology_filter
      if present_typologies.present?
        if all_is_present?
          {:typology.in => present_typologies}
        else
          {}
        end
      else
        {typology: {:$eq => nil}}
      end
    end
    
    def objectives_filters
      query = attributes.select do |key, val|
        FILTER_KEYS.include?(key.to_sym) && (val.present? || val.is_a?(FalseClass))
      end
      query.merge!(typology_filter)
      query.merge!(lessor_status_filter)
    end
    
    def old_offers_with(query)
      formated_query = query.each_with_object({}) do |(key, val), h|
        h["superfields.#{key}".to_sym] = val
      end
      offer_family = Family.find_by(slug: Hermes::ENV['offer_fam_slug'])
      formated_query[:family_id] = offer_family.id
      Tagset.where(formated_query)
    end
    
    def offers_with(query)
      case Hermes::ENV['transition_handler']
      when 'old'
        old_offers_with(query)
      else
        HousingOffer.where(query)
      end
    end
    
    def all_is_present?
      present_typologies.present? &&
        (present_typologies & [:all, 'all']).present?
    end
    
    def update_realized_typology(typology, offers)
      typology_filter = {typology: typology}
      if Hermes::ENV['transition_handler'] == 'old'
        typology_filter = typology_filter.each_with_object({}) do |(key, val), h|
          h["superfields.#{key}"] = val
        end
      end
      offers_count = if %w[Integer Number].include?(offers.class.name)
                       offers
                     else
                       offers.where(typology_filter).count
                     end
      realized_by_typologies[typology.to_sym] = offers_count
    end
    
    def initialize_result
      offers = offers_with(objectives_filters)
      if all_is_present?
        realized_by_typologies[:all] = offers.count
      else
        present_typologies.each {|typology| update_realized_typology(typology, offers)}
      end
    end
    
    def update_objective_difference_by_typologies
      present_typologies.each do |typology|
        value = (target_by_typologies&.fetch(typology, 0).to_i || 0) - (realized_by_typologies&.fetch(typology, 0).to_i || 0)
        objective_difference_by_typologies[typology] = value
      end
    end
    
    def objectives_moved?
      target_by_typologies_changed? || realized_by_typologies_changed?
    end
    
    def before_save_actions
   
    end
    
    def before_create_actions
      handle_activation_date
      before_actions
    end
    
    def before_actions
      if target_by_typologies_changed?
        correct_target_by_typologies
        update_present_typologies
        initialize_result
      end
      update_objective_difference_by_typologies if objectives_moved?
      throw_error_if_same_exists
    
    end
    
    def increment_realized_for(typology, housing_offer)
      return false unless typology.is_typology?
      
      typology = 'all' if all_is_present?
      self.realized_by_typologies[typology] ||= 0
      self.realized_by_typologies[typology] += 1
      self.save
      lessor_objective_histories.create!(
        housing_offer_id: housing_offer.id
      )
    end
    
    class << self
      def filter_keys
        FILTER_KEYS
      end
    end
    
    def throw_error_if_same_exists
      find_request = []
      searchable_items = {
        department: department,
        district: district,
        threshold_type: threshold_type,
        priority_public: priority_public,
        new_offer: new_offer,
        operation_name: operation_name,
        agency_id: agency_id,
        lessor_id: lessor_id,
        present_typologies: present_typologies,
        activation_date: activation_date
      }.each do |key, val|
        find_request.push({key.to_sym => val})
      end

=begin
      if activation_date
        find_request.push({:activation_date => (activation_date.beginning_of_year..activation_date.end_of_year)})
      else
        find_request.push({:activation_date => {"$exists" => false}})
      end
=end
      
      if Hermes::LessorObjective.where("$and" => find_request).length > 0 && Hermes::LessorObjective.where("$and" => find_request).first.id != id
        
        self.errors.add(:base, "There is already a lessor_objective like this one : " + Hermes::LessorObjective.where("$and" => find_request).first.id)
        return false
      else
        return true
      end
    
    end
    
    def start_update_target_objectives_on_offers
      if (department_changed? ||
        district_changed? ||
        threshold_type_changed? ||
        priority_public_changed? ||
        new_offer_changed? ||
        operation_name_changed? ||
        agency_id_changed? ||
        target_by_typologies_changed?)
        
        UpdateTargetObjectivesJob.perform_later(lessor_id.to_s)
      end
    end
    
    def after_save_actions
      start_update_target_objectives_on_offers
    
    end
  end
end
