# frozen_string_literal: true

module Hermes
  class UserRight
    include Hermes::Concerns::CommonFieldsConcern
    field :right, type: String
    field :type, type: String
    field :title, type: String
    field :group_id, type: String
    field :user_email, type: String
    field :user_first_name, type: String
    field :user_last_name, type: String
    field :user_phone_number, type: String
    field :user_auth_roles, type: Array
    
    common_indexes *%i[
      right
      type
      title
      group_id
      user_email
      user_first_name
      user_last_name
      user_phone_number
      user_auth_roles
    ]

    belongs_to :entity, index: true, class_name: 'Hermes::Entity'
    belongs_to :lessor, index: true, class_name: 'Hermes::Lessor'
    belongs_to :user, index: true, class_name: 'User'

    validates_presence_of :right, :type
    validates_presence_of :entity_id, if: -> {type == 'entity'}
    validates_presence_of :lessor_id, if: -> {type == 'lessor'}
    validates_inclusion_of :right, in: ["reader", "user", "administrator"]
    validates_inclusion_of :type, in: ["entity", "lessor"]

    before_save :before_save_actions
    after_create :after_create_actions
    before_destroy :before_destroy_actions

    private

    def before_save_actions
      self.group_id = get_group_id.to_s
      self.title = (type == 'entity') ? entity.title : lessor.title
    end

    def after_create_actions
      add_right_to_user
    end

    def before_destroy_actions
      remove_right_user
    end

    def get_group_id
      rights_mapping = {
        reader: 'group_id',
        user: 'supervisor_group_id',
        administrator: 'manager_group_id',
      }

      entity = case type
      when 'entity'
        Hermes::Entity.find(entity_id)
      when 'lessor'
        Hermes::Lessor.find(lessor_id)
      end

      entity && entity[rights_mapping[right.to_sym]]
    end
    
  def add_right_to_user
    if get_group_id
      user.group_ids = (user.group_ids + [get_group_id]).uniq unless user.group_ids.include?(get_group_id)
      user.save! if user.group_ids_changed?
    end
  end
  
  def remove_right_user
      user.group_ids = user.group_ids - [get_group_id] if user.group_ids.include?(get_group_id)
      user.save! if user.group_ids_changed?
    end
  end
end
